@@ -1,11 +1,10 @@
-use clap::{crate_version, AppSettings, Clap};
+use clap::{crate_version, Parser};
 use custom_error::custom_error;
 use std::{io, io::prelude::*};
 
 /// Tiny HTTP client for the Black (blackd) Python code formatter
-#[derive(Clap)]
+#[derive(Parser)]
 #[clap(version = crate_version!())]
-#[clap(setting = AppSettings::ColoredHelp)]
 struct Opts {
     /// URL of blackd server
     #[clap(long, default_value = "http://localhost:45484")]
@@ -16,13 +15,13 @@ custom_error! {BlackdError
     Minreq{source: minreq::Error} = "{source}",
     Syntax{details: String} = "Syntax Error: {details}",
     Formatting{details: String} = "Formatting Error: {details}",
-    Unknown{status_code: i32, body: String} = "Unknown Error: {status_code}",
+    Unknown{status_code: i32, body: String} = "Unknown Error {status_code}: {body}",
 }
 
 fn main() {
     let opts: Opts = Opts::parse();
     let stdin = read_stdin();
-    let result = format(opts.url, stdin.unwrap());
+    let result = format(&opts.url, &stdin.unwrap_or_default());
     match result {
         Ok(v) => print!("{}", v),
         Err(e) => {
@@ -42,17 +41,17 @@ fn read_stdin() -> Result<String, Box<dyn std::error::Error + Send + Sync>> {
     Ok(buffer)
 }
 
-fn format(url: String, stdin: String) -> Result<String, BlackdError> {
+fn format(url: &str, stdin: &str) -> Result<String, BlackdError> {
     let resp = minreq::post(url)
         .with_header("X-Fast-Or-Safe", "fast")
         .with_header("Content-Type", "text/plain; charset=utf-8")
-        .with_body(stdin.as_str())
+        .with_body(stdin)
         .send()?;
 
     let body = resp.as_str()?.to_string();
     match resp.status_code {
-        200 => Ok(body),  // input was reformatted by Black
-        204 => Ok(stdin), // input is already well-formatted
+        200 => Ok(body),              // input was reformatted by Black
+        204 => Ok(stdin.to_string()), // input is already well-formatted
         400 => Err(BlackdError::Syntax { details: body }),
         500 => Err(BlackdError::Formatting { details: body }),
         _ => Err(BlackdError::Unknown {
@@ -79,7 +78,7 @@ mod tests {
             then.status(200).body(body);
         });
 
-        let result = format(server.url(""), "print('Hello World!')".to_string());
+        let result = format(&server.url(""), "print('Hello World!')");
 
         mock.assert();
         assert!(result.is_ok());
@@ -97,7 +96,7 @@ mod tests {
             then.status(204);
         });
 
-        let result = format(server.url(""), body.to_string());
+        let result = format(&server.url(""), body);
 
         mock.assert();
         assert!(result.is_ok());
@@ -113,7 +112,7 @@ mod tests {
                 .body("Cannot parse: 1:6: print('bad syntax'))");
         });
 
-        let result = format(server.url(""), "print('bad syntax'))".to_string());
+        let result = format(&server.url(""), "print('bad syntax'))");
 
         mock.assert();
         assert!(result.is_err());
@@ -132,7 +131,7 @@ mod tests {
                 .body("('EOF in multi-line statement', (2, 0))");
         });
 
-        let result = format(server.url(""), "print(('bad syntax')".to_string());
+        let result = format(&server.url(""), "print(('bad syntax')");
 
         mock.assert();
         assert!(result.is_err());
@@ -147,13 +146,16 @@ mod tests {
         let server = MockServer::start();
         let mock = server.mock(|when, then| {
             when.method("POST");
-            then.status(418);
+            then.status(418).body("message");
         });
 
-        let result = format(server.url(""), String::new());
+        let result = format(&server.url(""), "");
 
         mock.assert();
         assert!(result.is_err());
-        assert_eq!(result.unwrap_err().to_string(), "Unknown Error: 418");
+        assert_eq!(
+            result.unwrap_err().to_string(),
+            "Unknown Error 418: message"
+        );
     }
 }
